@@ -24,9 +24,10 @@ extern int llr(mpz_t N);
 extern int proth(mpz_t N);
 extern int is_proth_form(mpz_t N);
 
-int _GMP_BPSW(mpz_t n);
-int is_deterministic_miller_rabin_prime(mpz_t n);  /* assumes n is BPSW */
+extern int _GMP_BPSW(mpz_t n);
+extern int is_deterministic_miller_rabin_prime(mpz_t n);  /* assumes n is BPSW */
 extern int  is_miller_prime(mpz_t n, int assume_grh);
+extern int is_bpsw_dmr_prime(mpz_t n);
 
 extern int  _GMP_is_prime(mpz_t n);
 extern int  _GMP_is_prob_prime(mpz_t n);
