@@ -5,6 +5,7 @@
 #define __STDC_LIMIT_MACROS
 #include <stdint.h>
 #else
+  typedef __int32 int32_t;
   typedef unsigned __int32 uint32_t;
   typedef unsigned __int64 uint64_t;
 #endif
@@ -19,7 +20,7 @@
   #define UVCONST(x) ((unsigned long)x##UL)
   #define UVuf "lu"
   #define IVdf "ld"
-  #define croak(fmt,...)            { printf(fmt,##__VA_ARGS__); exit(3); }
+  #define croak(fmt,...)            do { printf(fmt,##__VA_ARGS__); exit(3); } while(0)
   #define New(id, mem, size, type)  mem = (type*) malloc((size)*sizeof(type))
   #define Newz(id, mem, size, type) mem = (type*) calloc(size, sizeof(type))
   #define Renew(mem, size, type)    mem =(type*)realloc(mem,(size)*sizeof(type))
