@@ -1,6 +1,7 @@
 #ifndef MPU_UTILITY_H
 #define MPU_UTILITY_H
 
+#include <math.h>
 #include <gmp.h>
 #include "ptypes.h"
 
@@ -11,6 +12,8 @@ extern gmp_randstate_t* get_randstate(void);
 extern void init_randstate(unsigned long seed);
 extern void clear_randstate(void);
 
+int is_primitive_root(mpz_t a, mpz_t b, int nprime);
+
 /* tdiv_r is faster, but we'd need to guarantee the input is positive */
 #define mpz_mulmod(r, a, b, n, t)  \
   do { mpz_mul(t, a, b); mpz_mod(r, t, n); } while (0)
@@ -19,8 +22,9 @@ extern void clear_randstate(void);
 extern int mpz_divmod(mpz_t r, mpz_t a, mpz_t b, mpz_t n, mpz_t t);
 
 /* s = sqrt(a) mod p */
-extern int sqrtmod(mpz_t s, mpz_t a, mpz_t p,
-                   mpz_t t, mpz_t t2, mpz_t b, mpz_t g); /* 4 temp variables */
+extern int sqrtmod(mpz_t s, mpz_t a, mpz_t p);
+extern int sqrtmod_t(mpz_t s, mpz_t a, mpz_t p,
+                     mpz_t t, mpz_t t2, mpz_t b, mpz_t g); /* 4 tmp variables */
 
 extern unsigned long modinverse(unsigned long a, unsigned long p);
 
@@ -135,4 +139,24 @@ static int is_perfect_square(UV n, UV* sqrtn)
 }
 #endif
 
+#ifdef FUNC_mpz_logn
+static double mpz_logn(mpz_t n)
+{
+  long exp;
+  double logn = mpz_get_d_2exp(&exp, n);
+  logn = log(logn) + (log(2) * exp);
+  return logn;
+}
+#endif
+
+#ifdef FUNC_mpz_log2
+static double mpz_log2(mpz_t n)
+{
+  long exp;
+  double logn = mpz_get_d_2exp(&exp, n);
+  logn = exp + log(logn)/log(2);
+  return logn;
+}
+#endif
+
 #endif
